"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Helm = void 0;
const fs = require("fs");
const os = require("os");
const path = require("path");
const randomstring = require("randomstring");
const exec_1 = require("./exec");
const logger_1 = require("./logger");
const helm_1 = require("./resultsParsers/helm");
const istioctl_1 = require("./resultsParsers/istioctl");
const kubeval_1 = require("./resultsParsers/kubeval");
class Helm {
    constructor() {
        this.helmBinary = process.env.HELM_BINARY
            ? process.env.HELM_BINARY
            : 'helm';
        this.command = `${this.helmBinary} template --namespace default release-name .`;
        this.files = [];
        this.sets = [];
        this.phase1 = [];
        this.phase2 = [];
        this.exec = new exec_1.Exec();
        this.logger = new logger_1.Logger({ namespace: 'helm' });
        this.phase1.push(new helm_1.HelmResultParser());
        if (kubeval_1.KubeValResultsParser.ENABLED) {
            this.phase2.push(new kubeval_1.KubeValResultsParser());
        }
        if (istioctl_1.IstioCtlResultsParser.ENABLED) {
            this.phase2.push(new istioctl_1.IstioCtlResultsParser());
        }
    }
    withValueFile(valueFile) {
        const pathToValueFile = path.join(process.cwd(), valueFile);
        this.files.push(pathToValueFile);
        return this;
    }
    set(key, value) {
        if (Array.isArray(value)) {
            this.sets.push(`${key}={${value.join(',')}}`);
        }
        else {
            this.sets.push(`${key}=${value}`);
        }
        return this;
    }
    async go(done) {
        let filename = null;
        try {
            let command = this.command;
            if (this.files.length > 0) {
                command = `${command} -f ${this.files.join(' -f ')}`;
            }
            if (this.sets.length > 0) {
                command = `${command} --set ${this.sets.join(' --set ')}`;
            }
            this.files = [];
            this.sets = [];
            const result = await this.exec.command(command, { throw: true });
            if (istioctl_1.IstioCtlResultsParser.ENABLED || kubeval_1.KubeValResultsParser.ENABLED) {
                filename = path.join(os.tmpdir(), randomstring.generate({
                    length: 20,
                    charset: 'alphabetic'
                }));
                fs.writeFileSync(filename, result.stdout);
            }
            await Promise.all(this.phase1.map(async (parser) => {
                this.logger.debug(`running results parser: ${parser.constructor.name}`);
                await parser.parse({ result });
            }));
            await Promise.all(this.phase2.map(async (parser) => {
                this.logger.debug(`running results parser: ${parser.constructor.name}`);
                await parser.parse({ result, onDisk: filename });
            }));
            if (done) {
                done();
            }
        }
        catch (ex) {
            this.logger.error(ex.message);
            if (filename) {
                fs.unlinkSync(filename);
            }
            if (done) {
                done(ex);
            }
        }
        finally {
            if (filename) {
                fs.unlinkSync(filename);
            }
        }
    }
}
exports.Helm = Helm;
